//+------------------------------------------------------------------+
//| 8:01 AM 5/11/2016                                      mutex.mqh |
//|                              Copyright  2016, William H. Roeder |
//|                                        mailto:WHRoeder@yahoo_com |
//+------------------------------------------------------------------+
/** @file
 * @brief Mutex using a global variable.                                      */
//+------------------------------------------------------------------+
#include "basic_types.mqh"                   // Noncopyable
//+------------------------------------------------------------------+
/// Mutex using a global variable.
class Mutex : Noncopyable{
 public:
   void     Mutex(const string& name) :                                 // Ctor
      mLocked(false), mName(name){     // Attempt to create if it doesn't exist.
      if(!GlobalVariableCheck(mName) ) GlobalVariableTemp(mName);
   }
   virtual ~Mutex(){                               release();  }        // Dtor
                                                               // No copy/assign
 public:                                                          // Methods
   bool     is_locked(void)                  const{return mLocked;             }
   void     lock(COUNT maxWait=120000);      // 120 sec for network.
   void     release(void);
 private:                                                         // Data
   string   mName;
   bool     mLocked;
};
//+- Mutex ----------------------------------------------------------+
//|                                                                  |
//+- Mutex ------------------------------------------------- Public -+
/** Lock the mutex. May throw @ref Timeout_Exception or @ref
 * Interrupted_Exception.                                                     */
bool     Mutex::lock(COUNT maxWait=120000){  // 120 sec for network.
   COUNT    startTime   = GetTickCount();    long     delay = 0.0;
   for(;!mLocked;){
      if(!IsTradeContextBusy()
      && GlobalVariableSetOnCondition(mName, 0, 1) ){ mLocked = true;   break; }
      }  // lock, for
      if(is_stopping() )                                                break;
      long  delay = GetTickCount() - startTime;
      if(delay <= 0) delay += UINT_MAX + 1;
      if(delay > maxWait)                                               break;
      Comment(StringFormat(
         "%s: Wait %g seconds until another expert finishes trading...",
         WindowExpertName(), delay / 1000.0) );
      WindowRedraw();
      Sleep(500);
   }
   if(delay != 0){
      Comment("");   RefreshRates();         // I slept.
   }
   return mLocked;
}  // lock
void     Mutex::release(void){
   if(mLocked){
      if(!GlobalVariableSet(mName, 0) )
         PrintFormat("%s: GlobalVariableSet('%s',0): Error #",
                     __FUNCTION__, mName, _LastError);
      mLocked  = false;
}  }
//+- Mutex ----------------------------------------------------------+
//+------------------------------------------------------------------+
